#define I386

#ifndef I386
#define AVR
#endif

#ifdef AVR
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <stdio.h>
#include <stdlib.h>
#include <inttypes.h>
#include <util/delay.h>
#else
#include "inttypes.h"
#endif

#define F_CPU 16000000UL



#include "lcdg.h"
#include "8_3x5.h"
#include "Sced_Disp.h"
#include "Sced.h"
#include "softtimer.h"
#ifdef AVR
#include "math.h"
#include "uart.h"
#include "Led.h"
//hagen
#include "gccutils.h"
//#include "linsin.h"
#endif

#define UART_BAUDRATE	9600


#define SLM1606_RED                 0x01
#define SLM1606_GREEN               0x10
#define SLM1606_YELLOW              0x11
#define SLM1606_OFF                 0x00

#define SLM1606M_ROW_PIXEL          16
#define SLM1606M_COL_PIXEL          16
#define DISPLAY_H                   4
#define DISPLAY_V                   5




#define SCED_IDX_COOD_X             0
#define SCED_IDX_COOD_Y             1

#define SCED_DAYS_X                 7
#define SCED_DAYS_Y                 6


#define SCED_MARK_FILL_RED          0
#define SCED_MARK_FILL_GREEN        1
#define SCED_MARK_FILL_YELLOW       2

#define SCED_MARK_FRAME_RED         3
#define SCED_MARK_FRAME_GREEN       4
#define SCED_MARK_FRAME_YELLOW      5


uint8_t X_Values[16]={
0,
1,
3,
4,
5,
6,
8,
9,
10,
11,
13,
14,
15,
16,
17,
18};

uint8_t Y_Values[16]={
25,
25,
25,
25,
24,
24,
24,
23,
23,
22,
22,
21,
20,
19,
19,
18};

float Sinus[8]={
0.0f,
0.104528463f,
0.207911691f,
0.309016994f,
0.406736643f,
0.5f,
0.587785252f,
0.669130606f};

float Cosinus[8]={
1.0f,
0.994521895f,
0.978147601f,
0.951056516f,
0.913545458f,
0.866025404f,
0.809016994f,
0.743144825f};



#ifdef AVR
uint8_t date_clock=0;
date_t date;
static    uint8_t sec=0,min=0,std=0,ampm=0;
#else
uint8_t date_clock=0;
date_t date;
static    uint8_t sec=0,min=0,std=0,ampm=0;
unsigned int cnt,row,col,round,toggle=0;
uint16_t rec_uart=0,point_x=0,point_y=0;
uint8_t date_clock_update=1,rec=0,onoff=0;
#endif

void Clk_Draw_Pointer (uint8_t x,uint8_t y,uint8_t len,uint8_t pos,uint8_t col)
{
    uint8_t cnt,pointer_sec;
    int8_t x_p,y_p;
    cnt=pos;
    pointer_sec=len;


    switch(cnt/15)
    {
        case 0 :
        {
            if((cnt%15)<=7)
            {
                x_p=(int8_t)(Sinus[(cnt%15)&0x07]*(float)pointer_sec);
                y_p=(int8_t)(Cosinus[(cnt%15)&0x07]*(float)pointer_sec);
            }
            else
            {
                y_p=(int8_t)(Sinus[((cnt%15)^0x07)&0x07]*(float)pointer_sec);
                x_p=(int8_t)(Cosinus[((cnt%15)^0x07)&0x07]*(float)pointer_sec);
            }
            break;
        }
        case 1 :
        {
            if((cnt%15)<=7)
            {
                y_p=(int8_t)(Sinus[(cnt%15)&0x07]*(float)pointer_sec)*(-1);
                x_p=(int8_t)(Cosinus[(cnt%15)&0x07]*(float)pointer_sec);
            }
            else
            {
                x_p=(int8_t)(Sinus[((cnt%15)^0x07)&0x07]*(float)pointer_sec);
                y_p=(int8_t)(Cosinus[((cnt%15)^0x07)&0x07]*(float)pointer_sec)*(-1);
            }
            break;
        }
        case 2 :
        {
            if((cnt%15)<=7)
            {
                x_p=(int8_t)(Sinus[(cnt%15)&0x07]*(float)pointer_sec)*(-1);
                y_p=(int8_t)(Cosinus[(cnt%15)&0x07]*(float)pointer_sec)*(-1);
            }
            else
            {
                y_p=(int8_t)(Sinus[((cnt%15)^0x07)&0x07]*(float)pointer_sec)*(-1);
                x_p=(int8_t)(Cosinus[((cnt%15)^0x07)&0x07]*(float)pointer_sec)*(-1);
            }
            break;
        }
        case 3 :
        {
            if((cnt%15)<=7)
            {
                y_p=(int8_t)(Sinus[(cnt%15)&0x07]*(float)pointer_sec);
                x_p=(int8_t)(Cosinus[(cnt%15)&0x07]*(float)pointer_sec)*(-1);
            }
            else
            {
                x_p=(int8_t)(Sinus[((cnt%15)^0x07)&0x07]*(float)pointer_sec)*(-1);
                y_p=(int8_t)(Cosinus[((cnt%15)^0x07)&0x07]*(float)pointer_sec);
            }
            break;
        }
    }
	if(date_clock)
    	GRF_drwline(x,y,x+x_p,y+y_p,col);

}
void Clk_Redraw_Face (void)
{
	unsigned int cnt=0;

    lcdg_fillbox(0,0,63,79, 0x00);

    //SLM1606_Update_Disable();

    lcdg_filledcircle(31,40,31,0x01);
    lcdg_filledcircle(31,40,29,0x00);
    cnt=11;
    while(cnt)
    {
        Clk_Draw_Pointer(31,40,29,cnt*5,0x01);
        cnt--;
    }
    Clk_Draw_Pointer(31,40,29,0,0x01);
    lcdg_filledcircle(31,40,26,0x00);

    //SLM1606_Update_Enable();
}


void Clk_Sec_Set (uint8_t sec_set)
{
    static uint8_t sec_last=0;
    //if(sec_set!=sec_last)
    {
        Clk_Draw_Pointer(31,40,25,sec_last,0x00);
        sec_last=sec_set;
        Clk_Draw_Pointer(31,40,25,sec_last,0x10);
    }
}
void Clk_Min_Set (uint8_t min_set)
{
    static uint8_t min_last=0;
    //if(min_set!=min_last)
    {
        Clk_Draw_Pointer(31,40,19,min_last,0x00);
        min_last=min_set;
        Clk_Draw_Pointer(31,40,19,min_last,0x11);
    }
    /*if(sec-1==min_set||sec+59==min_set||sec==min_set)
    {
        Clk_Draw_Pointer(31,40,19,min_last,0x11);
    }*/
}
void Clk_Hou_Set (uint8_t hou_set)
{
    static uint8_t hou_last=0;
    //if(hou_set!=hou_last)
    {
        Clk_Draw_Pointer(31,40,15,hou_last,0x00);
        hou_last=hou_set;
        Clk_Draw_Pointer(31,40,15,hou_last,0x01);
    }
}



void Clk_Cnt_Up_Min (void)
{
    if(min>=59)
    {
        min=0;
    }
    else
    {
        min++;
    }
    Clk_Min_Set(min);
}
void Clk_Cnt_Down_Min (void)
{
    if(min==0)
    {
        min=59;
    }
    else
    {
        min--;
    }
    Clk_Min_Set(min);
}

void Clk_Cnt_Up_Hou (void)
{
    if(std>=11)
    {
        std=0;
    }
    else
    {
        std++;
    }
    Clk_Hou_Set(std*5+(min/12));
}

void Clk_Cnt_Down_Hou (void)
{
    if(std==0)
    {
        std=11;
    }
    else
    {
        std--;
    }
    Clk_Hou_Set(std*5+(min/12));
}

void Clk_Hou_AmPm_Toggle (uint8_t run)
{
    ampm++;
    if(ampm==2)
    {
        ampm=0;
        //sould an dayincrement be done
        if(run)
            Date_Count_Up_Day(&date);
        //is date displaying active
        if(date_clock==0)
            Sced_Disp_Show_Date(&date,SCED_DISP_UPDATE_ALL);
    }
    if(!run)
    {
        if(ampm)
        {
            lcdg_drawpixel(31, 40, 0x10);
        }
        else
        {
          
            lcdg_drawpixel(31, 40, 0x11);
        }
    }
}

void Clk_Clk(void)
{

    
    sec++;
    if(sec>59)
    {
      sec=0;
      min++;
      if(min>59)
      {
        min=0;
        std++;
        if(std>11)
        {
          std=0;
          Clk_Hou_AmPm_Toggle(1);
        }
      }
    }
    Clk_Sec_Set(sec);
    Clk_Min_Set(min);
    Clk_Hou_Set(std*5+(min/12));
}


#ifdef AVR
int main (void)
{
	unsigned int cnt,row,col,round,toggle=0;
    uint16_t rec_uart=0,point_x=0,point_y=0;
    uint8_t date_clock_update=1,rec=0,onoff=0;



	TESTLED_LED_OUT;
	LED_RED_OUT;
	LED_GREEN_OUT;
	LED_YELLOW_OUT;

    DDRA=0xFF;
    DDRC=0xFF;

    //switch on the fans
    DDRE=0x04;
    PORTE=0x04;//switch on the fans


    //IR PORT E 4
    DDRE&=0xEF;//port E4 to input
    //PORTE|=0x10;//switch on the pull up -> pull up is not nessersary
    //

    PORTA=0x00;//select all colour lines to 0
    PORTC=0x5F;//select all modules on 0x1f make bright off 0x40
    //reset
    PORTC=PORTC|0x80;
    PORTC=PORTC&0x7f;

    //clock all displays with 0x00
    cnt=0;
	while(cnt<256)
	{
        PORTC=PORTC|0x20;
        PORTC=PORTC&0xdf;

        cnt++;

	}


    SLM1606_Update_Enable();
    //code

    //reset
    PORTC=PORTC|0x80;
    _delay_ms(1);

    PORTC=PORTC&0x7f;
    _delay_ms(1);

    PORTA=0x00;
    PORTC=0x00;
    //4
    //2
    //1
    //8
    //10



    sei();

    uart_init(UART_BAUD_SELECT(UART_BAUDRATE,F_CPU));  
#else

int main_ini (void)
{
#endif

    Sced_Disp_Init();

    Softtimer__Init();
    
    
    Softtimer__Load(1,0);
    Softtimer__Load(2,0);
    Softtimer__Load(3,0);

    date.day=13;//+cnt;
    date.month=2;
    date.year=2010;

#ifdef AVR
    cnt=0;
    while(1)
    {

#else
}
int main_prog (void)
{
    cnt=0;
    if(1)
    {

#endif
        Softtimer__Handler();

        if(Softtimer__Expired(1))
        {
            Softtimer__Load(1,995);
            
            Clk_Clk();
        }
        if(Softtimer__Expired(2))
        {
            Softtimer__Load(2,500);
            /*if(cnt<=20)
            {
                date.day=21;//+cnt;
                date.month=10;
                date.year=2009;
                Sced_Disp_Show_Date(&date);
                cnt++;
            }*/


        }
        if(Softtimer__Expired(3))
        {
            Softtimer__Load(3,500);
            //verhindert ein update wenn schon auf clock umgeschaltet wurde und der cursor noch an war
            if(!date_clock)
                Sced_Disp_Cursor_Toggle();
        }
        //IR Settings
        /*if(PINE&0x10)
        {
            cnt%=30;
        }
        else
        {
            //cnt++;
            cnt%=30;
        }*/
		if(date_clock_update)
		{
            date_clock_update=0;
            lcdg_fillbox(0,0,63,79, 0x00);
			if(date_clock)
            {
                Sced_Disp_Cursor_Set(0);
                Clk_Redraw_Face();
            }
			else
			{
                Sced_Redraw_Face();
                Sced_Disp_Show_Date(&date,SCED_DISP_UPDATE_ALL);                              
			}
		}
        rec_uart=uart_getc();
		if (!(rec_uart&0x0100))					// Daten im UART Empfangspuffer ?
		{
            rec=(uint8_t)rec_uart;
            switch(rec)
            {
                case '7':
                {
                    date_clock^=0x01;
                    date_clock_update=1;
                    onoff=0;
                }
                break;
            }
            if(date_clock)
            {
                /*clock*/
                switch(rec)
                {
                    case '2':
                    {
                        Clk_Cnt_Down_Hou();
                    }
                    break; 
                    case '3':
                    {
                        Clk_Cnt_Down_Min();
                    }
                    break;
                    case '5':
                    {
                        Clk_Hou_AmPm_Toggle(0);
                    }
                    break;
                    case '8':
                    {
                        Clk_Cnt_Up_Hou();
                    }
                    break;
                    case '9':
                    {
                        Clk_Cnt_Up_Min();
                    }
                    break;
                }
            }
            else
            {
                /*date*/
                switch(rec)
                {
                    case '1':
                    {
                        onoff^=0xFF;
                        Sced_Disp_Cursor_Set(onoff);
                        Softtimer__Load(3,0);
                    }
                    break;
                }

                if(onoff)
                {
                    switch(rec)
                    {                    
                        case '8':
                        case '6':
                        case '2':
                        case '4':
                        {
                            Sced_Disp_Cursor_Set_Pos(rec,&date);
                            Softtimer__Load(3,0);
                        }
                        break;
                        case '5':
                        {
                            if(date.day!=Sced_Disp_Cursor_Get_Pos())
                            {
                                date.day=Sced_Disp_Cursor_Get_Pos();
                                Sced_Disp_Show_Date(&date,SCED_DISP_UPDATE_ALL_NESS);
                            }
                        }
                        break;
                    }
                }
            }         
        }
	}
}

