

#ifndef SCED_DISP_H
#define SCED_DISP_H

#include <inttypes.h>
#include "date.h"


#define SCED_DISP_DAYS_X                7
#define SCED_DISP_DAYS_Y                6
/*
   :                      :
   | .................... |
   |                      |
   |                    1 |  0
   |                      |
   |  2  3  4  5  6  7  8 |  1
   |                      |
   |  9 10 11 12 13 14 15 |  2
   |                      |    SCED_DISP_DAYS_Y
   | 16 17 18 19 20 21 22 |  3
   |                      |
   | 23 24 25 26 27 28 29 |  4
   |                      |
   | 30 31                |  5
   |..................... |
   :                      :

      0  1  2  3  4  5  6

      SCED_DISP_DAYS_X


*/
//draw colors

#define SCED_DISP_COL_OFF               0x00
#define SCED_DISP_COL_RED               0x01
#define SCED_DISP_COL_YELLOW            0x03
#define SCED_DISP_COL_GREEN             0x02


                               /*   .----------------------.   */
#define SCED_DISP_COL_MONTH    /*   |   S e p t e m b e r  |<- */   SCED_DISP_COL_GREEN
                               /*   |                      |   */
#define SCED_DISP_COL_YEAR     /*   |        2 0 0 9       |<- */   SCED_DISP_COL_GREEN
#define SCED_DISP_COL_WDLINE   /*   | .................... |<- */   SCED_DISP_COL_RED
#define SCED_DISP_COL_SCALE    /*   | Mo Di Mi Do Fr Sa So |<- */   SCED_DISP_COL_RED
                               /*   | .................... |<- */
                               /*   |                      |   */
                               /*   |     1  2  3  4  5  6 |   */
                               /*   |            .--.      |   */
                               /*   |  7  8  9 10|11|12 13 |   */
                               /*   |            '--'      |   */
                               /*   | 14 15 16 17 18 19 20 |   */
                               /*   |         .  .         |   */
#define SCED_DISP_COL_CURSOR   /*   | 16 17 18 19 20 21 22 |<- */   SCED_DISP_COL_RED
                               /*   |         '  '         |   */
                               /*   | 23 24 25 26 27 28 29 |   */
                               /*   |                      |   */
                               /*   | 30 31                |   */
                               /*                              */
                               /*     |  |  |  |  |  |  |      */
#define SCED_DISP_COL_SUNDAY   /*     |  |  |  |  |  |  '---<- */   SCED_DISP_COL_RED
#define SCED_DISP_COL_SATURDAY /*     |  |  |  |  |  '------<- */   SCED_DISP_COL_YELLOW
#define SCED_DISP_COL_WEEKDAY  /*     '--'--'--'--'---------<- */   SCED_DISP_COL_GREEN
                               /*                              */
#define SCED_DISP_COL_CWLINE   /*   |..................... |<- */   SCED_DISP_COL_RED
                               /*   |KW: 36 37 38 39 40    |   */
                               /*   '----------------------'   */
                               /*                              */
                               /*     |  |  |  |  |  |  |      */
#define SCED_DISP_COL_CALWEEK  /*     |  '--'--'--'--'--'---<- */   SCED_DISP_COL_RED
#define SCED_DISP_COL_CWSCALE  /*     '---------------------<- */   SCED_DISP_COL_RED





//Marc colors

#define SCED_DISP_MARCCOL_RED           0x01
#define SCED_DISP_MARCCOL_YELLOW        0x02
#define SCED_DISP_MARCCOL_GREEN         0x04


#define SCED_DISP_MARCCOL_CW            SCED_DISP_MARCCOL_GREEN


typedef struct
{
    uint8_t  Day         :6,
             Day_Color   :2;
    uint8_t  Frame_High  :1,
             Frame_Mid   :1,
             Frame_Low   :1,
             Fill_High   :1,
             Fill_Mid    :1,
             Fill_Low    :1,
             Cursor      :1,
             Update      :1;
}Sced_Disp_Day_t;

typedef struct
{
     uint8_t Frame_High  :1,//0
             Frame_Mid   :1,
             Frame_Low   :1,
             Fill_High   :1,
             Fill_Mid    :1,
             Fill_Low    :1,
             Cursor      :1,
             Update      :1;//7
}Sced_Disp_Marc_t;

//Marc Mask funtion defines*/
#define SCED_DISP_MARC_MASK_FRAME_H     0x01
#define SCED_DISP_MARC_MASK_FRAME_M     0x02
#define SCED_DISP_MARC_MASK_FRAME_L     0x04
#define SCED_DISP_MARC_MASK_FILL_H      0x08
#define SCED_DISP_MARC_MASK_FILL_M      0x10
#define SCED_DISP_MARC_MASK_FILL_L      0x20
#define SCED_DISP_MARC_MASK_CURSOR      0x40

#define SCED_DISP_UPDATE_ALL            0x80
#define SCED_DISP_UPDATE_ALL_NESS       0x40
#define SCED_DISP_UPDATE_DAY            0x01
#define SCED_DISP_UPDATE_MONTH          0x02
#define SCED_DISP_UPDATE_YEAR           0x04
#define SCED_DISP_UPDATE_WEEK           0x08

typedef struct
{
    Sced_Disp_Day_t Day[SCED_DISP_DAYS_X];
}Sced_Disp_Week_t;

typedef struct
{
    Sced_Disp_Week_t Week[SCED_DISP_DAYS_Y];
}Sced_Disp_Weeks_t;
#ifdef __cplusplus
extern "C"
{
#endif
void Sced_Disp_Set_Weeks_Pixel_Pos(uint8_t xpos, uint8_t ypos);
void Sced_Disp_Set_CalWeek_Pixel_Pos(uint8_t xpos, uint8_t ypos);


void Sced_Disp_Init(void);
void Sced_Redraw_Face (void);
void Sced_Disp_Show_Date(date_t *,uint8_t);
void Sced_Disp_Draw_Day(unsigned char x, unsigned char y,Sced_Disp_Day_t*);

void Sced_Disp_Draw_Number(uint8_t x, uint8_t y, uint8_t number,uint8_t mode, uint8_t color,uint8_t bgcolor);

void Sced_Disp_Cursor_Toggle(void);
void Sced_Disp_Cursor_Set(uint8_t val);
void Sced_Disp_Cursor_Set_Pos(uint8_t pos,date_t*);
uint8_t Sced_Disp_Cursor_Get_Pos (void);

#ifdef __cplusplus
}
#endif

#endif //SCED_DISP_H
